


class Card:
    def __init__(self,
                 pan_fragment: str = None,
                 type: str = None,
                 ):
        self.pan_fragment = pan_fragment
        self.type = type






class BalanceDetails:
    def __init__(self,
                 total: float = None,
                 available: float = None,
                 deposition_pending: float = None,
                 blocked: float = None,
                 debt: float = None,
                 hold: float = None,
                 ):

        self.total = total

        self.available = available

        self.deposition_pending = deposition_pending

        self.blocked = blocked

        self.debt = debt

        self.hold = hold







from datetime import datetime


class Operation:
    def __init__(self,
                 operation_id: str = None,
                 status: str = None,
                 datetime: datetime = None,
                 title: str = None,
                 pattern_id: str = None,
                 direction: str = None,
                 amount: float = None,
                 label: str = None,
                 type: str = None,
                 json: dict = {}
                 ):
        self.operation_id = operation_id
        self.status = status
        self.datetime = datetime
        self.title = title
        self.pattern_id = pattern_id
        self.direction = direction
        self.amount = amount
        self.label = label
        self.type = type
        self.json = json


from typing import List

from yoomoney_async.operation_details.digital_product import DigitalProduct
from yoomoney_async.operation_details.digital_bonus import DigitalBonus



class DigitalGood:
    def __init__(self,
                 products: List[DigitalProduct],
                 bonuses: List[DigitalBonus]
                 ):
        self.products = products
        self.bonuses = bonuses



from typing import List
import requests

from yoomoney_async.exceptions import (
    InvalidRequest,
    UnauthorizedClient,
    InvalidGrant,
    EmptyToken
    )

class Authorize:
    def __init__(
            self,
            client_id: str,
            redirect_uri: str,
            scope: List[str]
                  ):

        url = "https://yoomoney.ru/oauth/authorize?client_id={client_id}&response_type=code" \
              "&redirect_uri={redirect_uri}&scope={scope}".format(client_id=client_id,
                                                                  redirect_uri=redirect_uri,
                                                                  scope='%20'.join([str(elem) for elem in scope]),
                                                                  )

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        response = requests.request("POST", url, headers=headers)

        if response.status_code == 200:
            print("Visit this website and confirm the application authorization request:")
            print(response.url)

        code = str(input("Enter redirected url (https://yourredirect_uri?code=XXXXXXXXXXXXX) or just code: "))
        try:
            code = code[code.index("code=") + 5:].replace(" ","")
        except:
            pass

        url = "https://yoomoney.ru/oauth/token?code={code}&client_id={client_id}&" \
              "grant_type=authorization_code&redirect_uri={redirect_uri}".format(code=str(code),
                                                                                 client_id=client_id,
                                                                                 redirect_uri=redirect_uri,
                                                                                 )

        response = requests.request("POST", url, headers=headers)

        if "error" in response.json():
            error = response.json()["error"]
            if error == "invalid_request":
                raise InvalidRequest()
            elif error == "unauthorized_client":
                raise UnauthorizedClient()
            elif error == "invalid_grant":
                raise InvalidGrant()

        if response.json()['access_token'] == "":
            raise EmptyToken()

        print("Your access token:")
        print(response.json()['access_token'])


from datetime import datetime
from typing import Optional, List
import requests

from yoomoney_async.exceptions import (
    IllegalParamOperationId,
    TechnicalError
    )

from yoomoney_async.operation_details.digital_product import DigitalProduct
from yoomoney_async.operation_details.digital_bonus import DigitalBonus
from yoomoney_async.operation_details.digital_good import DigitalGood


class OperationDetails:
    def __init__(self,
                 base_url: str,
                 token: str,
                 operation_id: str,
                 method: str = None,
                 ):
        self.__private_method = method
        self.__private_token = token
        self.__private_base_url = base_url
        self.operation_id = operation_id

        data = self._request()

        if "error" in data:
            if data["error"] == "illegal_param_operation_id":
                raise IllegalParamOperationId()
            else:
                raise TechnicalError()

        self.status = None
        self.pattern_id = None
        self.direction = None
        self.amount = None
        self.amount_due = None
        self.fee = None
        self.datetime = None
        self.title = None
        self.sender = None
        self.recipient = None
        self.recipient_type = None
        self.message = None
        self.comment = None
        self.codepro = None
        self.protection_code = None
        self.expires = None
        self.answer_datetime = None
        self.label = None
        self.details = None
        self.type = None
        self.digital_goods = None

        if "status" in data:
            self.status = data["status"]
        if "pattern_id" in data:
            self.pattern_id = data["pattern_id"]
        if "direction" in data:
            self.direction = data["direction"]
        if "amount" in data:
            self.amount = data["amount"]
        if "amount_due" in data:
            self.amount_due = data["amount_due"]
        if "fee" in data:
            self.fee = data["fee"]
        if "datetime" in data:
            self.datetime = datetime.strptime(str(data["datetime"]).replace("T", " ").replace("Z", ""), '%Y-%m-%d %H:%M:%S')
        if "title" in data:
            self.title = data["title"]
        if "sender" in data:
            self.sender = data["sender"]
        if "recipient" in data:
            self.recipient = data["recipient"]
        if "recipient_type" in data:
            self.recipient_type = data["recipient_type"]
        if "message" in data:
            self.message = data["message"]
        if "comment" in data:
            self.comment = data["comment"]
        if "codepro" in data:
            self.codepro = bool(data["codepro"])
        if "protection_code" in data:
            self.protection_code = data["protection_code"]
        if "expires" in data:
            self.datetime = datetime.strptime(str(data["expires"]).replace("T", " ").replace("Z", ""), '%Y-%m-%d %H:%M:%S')
        if "answer_datetime" in data:
            self.datetime = datetime.strptime(str(data["answer_datetime"]).replace("T", " ").replace("Z", ""), '%Y-%m-%d %H:%M:%S')
        if "label" in data:
            self.label = data["label"]
        if "details" in data:
            self.details = data["details"]
        if "type" in data:
            self.type = data["type"]
        if "digital_goods" in data:
            products: List[DigitalProduct] = []
            for product in data["digital_goods"]["article"]:
                digital_product = DigitalProduct(merchant_article_id=product["merchantArticleId"],
                                                 serial=product["serial"],
                                                 secret=product["secret"],
                                                 )
                products.append(digital_product)

            bonuses: List[DigitalBonus] = []
            for bonus in data["digital_goods"]["bonus"]:
                digital_product = DigitalBonus(serial=bonus["serial"],
                                               secret=bonus["secret"],
                                               )
                bonuses.append(digital_product)

            self.digital_goods = DigitalGood(products=products,
                                             bonuses=bonuses
                                             )

    def _request(self):

        access_token = str(self.__private_token)
        url = self.__private_base_url + self.__private_method

        headers = {
            'Authorization': 'Bearer ' + str(access_token),
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        payload = {}

        payload["operation_id"] = self.operation_id


        response = requests.request("POST", url, headers=headers, data=payload)

        return response.json()



from datetime import datetime
from typing import Optional
import aiohttp
import json
from yoomoney_async.operation.operation import Operation

from .exceptions import (
    IllegalParamType,
    IllegalParamStartRecord,
    IllegalParamRecords,
    IllegalParamLabel,
    IllegalParamFromDate,
    IllegalParamTillDate,
    TechnicalError
)


class History:
    def __init__(self,
                 base_url: str = None,
                 token: str = None,
                 method: str = None,
                 type: str = None,
                 label: str = None,
                 from_date: Optional[datetime] = None,
                 till_date: Optional[datetime] = None,
                 start_record: str = None,
                 records: int = None,
                 details: bool = None,
                 ):

        self.__private_method = method

        self.__private_base_url = base_url
        self.__private_token = token

        self.type = type
        self.label = label
        try:
            if from_date is not None:
                from_date = "{Y}-{m}-{d}T{H}:{M}:{S}+03:00".format(
                    Y=str(from_date.year),
                    m=str(from_date.month),
                    d=str(from_date.day),
                    H=str(from_date.hour),
                    M=str(from_date.minute),
                    S=str(from_date.second)
                )
        except Exception:
            raise IllegalParamFromDate()

        try:
            if till_date is not None:
                till_date = "{Y}-{m}-{d}T{H}:{M}:{S}+03:00".format(
                    Y=str(till_date.year),
                    m=str(till_date.month),
                    d=str(till_date.day),
                    H=str(till_date.hour),
                    M=str(till_date.minute),
                    S=str(till_date.second)
                )
        except Exception:
            IllegalParamTillDate()

        self.from_date = from_date
        self.till_date = till_date
        self.start_record = start_record
        self.records = records
        self.details = details
        self.operations = list()

    async def start(self):
        data = await self._request()

        if "error" in data:
            if data["error"] == "illegal_param_type":
                raise IllegalParamType()
            elif data["error"] == "illegal_param_start_record":
                raise IllegalParamStartRecord()
            elif data["error"] == "illegal_param_records":
                raise IllegalParamRecords()
            elif data["error"] == "illegal_param_label":
                raise IllegalParamLabel()
            elif data["error"] == "illegal_param_from":
                raise IllegalParamFromDate()
            elif data["error"] == "illegal_param_till":
                raise IllegalParamTillDate()
            else:
                raise TechnicalError()

        for operation_data in data["operations"]:
            param = {}
            if "operation_id" in operation_data:
                param["operation_id"] = operation_data["operation_id"]
            else:
                param["operation_id"] = None
            if "status" in operation_data:
                param["status"] = operation_data["status"]
            else:
                param["status"] = None
            if "datetime" in operation_data:
                param["datetime"] = datetime.strptime(
                    str(operation_data["datetime"]).replace("T", " ").replace("Z", ""), '%Y-%m-%d %H:%M:%S')
            else:
                param["datetime"] = None
            if "title" in operation_data:
                param["title"] = operation_data["title"]
            else:
                param["title"] = None
            if "pattern_id" in operation_data:
                param["pattern_id"] = operation_data["pattern_id"]
            else:
                param["pattern_id"] = None
            if "direction" in operation_data:
                param["direction"] = operation_data["direction"]
            else:
                param["direction"] = None
            if "amount" in operation_data:
                param["amount"] = operation_data["amount"]
            else:
                param["amount"] = None
            if "label" in operation_data:
                param["label"] = operation_data["label"]
            else:
                param["label"] = None
            if "type" in operation_data:
                param["type"] = operation_data["type"]
            else:
                param["type"] = None

            operation = Operation(
                operation_id=param["operation_id"],
                status=param["status"],
                datetime=datetime.strptime(str(param["datetime"]).replace("T", " ").replace("Z", ""),
                                           '%Y-%m-%d %H:%M:%S'),
                title=param["title"],
                pattern_id=param["pattern_id"],
                direction=param["direction"],
                amount=param["amount"],
                label=param["label"],
                type=param["type"],
                json=operation_data,
            )
            self.operations.append(operation)
        return self

    async def _request(self):
        access_token = str(self.__private_token)
        url = self.__private_base_url + self.__private_method

        headers = {
            'Authorization': 'Bearer ' + str(access_token),
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        payload = {}
        if self.type is not None:
            payload["type"] = self.type
        if self.label is not None:
            payload["label"] = self.label
        if self.from_date is not None:
            payload["from"] = self.from_date
        if self.till_date is not None:
            payload["till"] = self.till_date
        if self.start_record is not None:
            payload["start_record"] = self.start_record
        if self.records is not None:
            payload["records"] = self.records
        if self.details is not None:
            payload["details"] = self.details

        async with aiohttp.ClientSession() as client:
            async with client.post(url, headers=headers, data=payload) as response:
                result = await response.text()

        return json.loads(result)


import requests
import json
from typing import TYPE_CHECKING, Optional, List

from yoomoney_async.account.balance_details import BalanceDetails
from yoomoney_async.account.card import Card
from yoomoney_async.exceptions import InvalidToken

class Account:

    def __init__(self,
                 base_url: str = None,
                 token: str = None,
                 method: str = None,

                 ):

        self.__private_method = method

        self.__private_base_url = base_url
        self.__private_token = token

        data = self._request()

        if len(data) != 0:
            self.account = data['account']
            self.balance = data['balance']
            self.currency = data['currency']
            self.account_status = data['account_status']
            self.account_type = data['account_type']

            self.balance_details = BalanceDetails()
            if 'balance_details' in data:
                if 'available' in data['balance_details']:
                    self.balance_details.available = float(data['balance_details']['available'])
                if 'blocked' in data['balance_details']:
                    self.balance_details.blocked = float(data['balance_details']['blocked'])
                if 'debt' in data['balance_details']:
                    self.balance_details.debt = float(data['balance_details']['debt'])
                if 'deposition_pending' in data['balance_details']:
                    self.balance_details.deposition_pending = float(data['balance_details']['deposition_pending'])
                if 'total' in data['balance_details']:
                    self.balance_details.total = float(data['balance_details']['total'])
                if 'hold' in data['balance_details']:
                    self.balance_details.hold = float(data['balance_details']['hold'])

            self.cards_linked = []
            if 'cards_linked' in data:
                for card_linked in data['cards_linked']:
                    card = Card(pan_fragment=card_linked['pan_fragment'], type=card_linked['type'])
                    self.cards_linked.append(card)
        else:
            raise InvalidToken()

    def _request(self):

        access_token = str(self.__private_token)
        url = self.__private_base_url + self.__private_method

        headers = {
            'Authorization': 'Bearer ' + str(access_token),
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        response = requests.request("POST", url, headers=headers)

        return response.json()





class DigitalProduct:
    def __init__(self,
                 merchant_article_id: str,
                 serial: str,
                 secret: str
                 ):
        self.merchant_article_id = merchant_article_id
        self.serial = serial
        self.secret = secret



import requests
import aiohttp

class Quickpay:
    def __init__(self,
                 receiver: str,
                 quickpay_form: str,
                 targets: str,
                 paymentType: str,
                 sum: float,
                 formcomment: str = None,
                 short_dest: str = None,
                 label: str = None,
                 comment: str = None,
                 successURL: str = None,
                 need_fio: bool = None,
                 need_email: bool = None,
                 need_phone: bool = None,
                 need_address: bool = None,
                 base_url: str = None,
                 response: dict = None,
                 redirected_url: str = None
                 ):
        self.receiver = receiver
        self.quickpay_form = quickpay_form
        self.targets = targets
        self.paymentType = paymentType
        self.sum = sum
        self.formcomment = formcomment
        self.short_dest = short_dest
        self.label = label
        self.comment = comment
        self.successURL = successURL
        self.need_fio = need_fio
        self.need_email = need_email
        self.need_phone = need_phone
        self.need_address = need_address
        self.base_url = base_url
        self.response = response
        self.redirected_url = redirected_url

    async def start(self):
        self.base_url = "https://yoomoney.ru/quickpay/confirm.xml?"

        payload = {"receiver": self.receiver, "quickpay_form": self.quickpay_form, "targets": self.targets,
                   "paymentType": self.paymentType, "sum": self.sum}

        if self.formcomment is not None:
            payload["formcomment"] = self.formcomment
        if self.short_dest is not None:
            payload["short_dest"] = self.short_dest
        if self.label is not None:
            payload["label"] = self.label
        if self.comment is not None:
            payload["comment"] = self.comment
        if self.successURL is not None:
            payload["successURL"] = self.successURL
        if self.need_fio is not None:
            payload["need_fio"] = self.need_fio
        if self.need_email is not None:
            payload["need_email"] = self.need_email
        if self.need_phone is not None:
            payload["need_phone"] = self.need_phone
        if self.need_address is not None:
            payload["need_address"] = self.need_address

        for value in payload:
            self.base_url += str(value).replace("_", "-") + "=" + str(payload[value])
            self.base_url += "&"

        self.base_url = self.base_url[:-1].replace(" ", "%20")
        async with aiohttp.ClientSession() as client:
            async with await client.post(self.base_url) as response:
                self.response = response
        self.redirected_url = self.response.url
        return self

async def payment_link(receiver: str, targets: str, price: int, label: str):
    quick_pay = await Quickpay(
        receiver=receiver,
        quickpay_form="shop",
        targets=targets,
        paymentType="SB",
        sum=price,
        label=label
    ).start()
    return quick_pay.redirected_url
    #return quick_pay.base_url




class YooMoneyError(Exception):
    """Basic class"""


class InvalidToken(YooMoneyError):

    message = "Token is not valid, or does not have the appropriate rights"
    def __init__(self, ):
        super().__init__(self.message)


class IllegalParamType(YooMoneyError):

    message = "Invalid parameter value 'type'"
    def __init__(self, ):
        super().__init__(self.message)

class IllegalParamStartRecord(YooMoneyError):

    message = "Invalid parameter value 'start_record'"
    def __init__(self, ):
        super().__init__(self.message)


class IllegalParamRecords(YooMoneyError):

    message = "Invalid parameter value 'records'"
    def __init__(self, ):
        super().__init__(self.message)

class IllegalParamLabel(YooMoneyError):

    message = "Invalid parameter value 'label'"
    def __init__(self, ):
        super().__init__(self.message)

class IllegalParamFromDate(YooMoneyError):

    message = "Invalid parameter value 'from_date'"
    def __init__(self, ):
        super().__init__(self.message)


class IllegalParamTillDate(YooMoneyError):

    message = "Invalid parameter value 'till_date'"
    def __init__(self, ):
        super().__init__(self.message)

class IllegalParamOperationId(YooMoneyError):

    message = "Invalid parameter value 'operation_id'"
    def __init__(self, ):
        super().__init__(self.message)

class TechnicalError(YooMoneyError):

    message = "Technical error, try calling the operation again later"
    def __init__(self, ):
        super().__init__(self.message)

class InvalidRequest(YooMoneyError):

    message = "Required query parameters are missing or have incorrect or invalid values"
    def __init__(self, ):
        super().__init__(self.message)

class UnauthorizedClient(YooMoneyError):

    message = "Invalid parameter value 'client_id' or 'client_secret', or the application" \
              " does not have the right to request authorization (for example, YooMoney blocked it 'client_id')"
    def __init__(self, ):
        super().__init__(self.message)

class InvalidGrant(YooMoneyError):

    message = "In issue 'access_token' denied. YuMoney did not issue a temporary token, " \
              "the token is expired, or this temporary token has already been issued " \
              "'access_token' (repeated request for an authorization token with the same temporary token)"
    def __init__(self, ):
        super().__init__(self.message)

class EmptyToken(YooMoneyError):

    message = "Response token is empty. Repeated request for an authorization token"
    def __init__(self, ):
        super().__init__(self.message)


from setuptools import setup, find_packages, Extension

setup(
    name='YooMoney_async',
    version='0.0.1',
    packages=find_packages(),
    url='https://github.com/Navigatoroleg/yoomoney_asunc',
    author='OlesMokovoz',
    author_email='oleg-oleg-2023@mail.ru',
    description='Unofficial YooMoney_async API python library'
)